@@ -34,37 +34,61 @@ if(max(Model2.order)>d || min(Model2.order)<1) stop("Order vector of model 2 is
 
 for(i in 1:(d*(d-1)/2))
 	{
-	if(!(Model1.family[i] %in% c(0,1,2,3,4,5,6,7,9,13,14,16,23,24,26,33,34,36))) stop("Copula family not implemented (in Model1.family).")
+	if(!(Model1.family[i] %in% c(0,1:10,13,14,16:20,23,24,26:30,33,34,36:40))) stop("Copula family not implemented (in Model1.family).")
   	if((Model1.family[i]==1 || Model1.family[i]==2) && abs(Model1.par[i])>=1) stop("The parameter of the Gaussian and t-copula has to be in the interval (-1,1).")
 	if(Model1.family[i]==2 && Model1.par2[i]<=1) stop("The degrees of freedom parameter of the t-copula has to be larger than 1.")
   	if((Model1.family[i]==3 || Model1.family[i]==13) && Model1.par[i]<=0) stop("The parameter of the Clayton copula has to be positive.")
   	if((Model1.family[i]==4 || Model1.family[i]==14) && Model1.par[i]<1) stop("The parameter of the Gumbel copula has to be in the interval [1,oo).")
   	if((Model1.family[i]==6 || Model1.family[i]==16) && Model1.par[i]<=1) stop("The parameter of the Joe copula has to be in the interval (1,oo).")	
   	if(Model1.family[i]==5 && Model1.par[i]==0) stop("The parameter of the Frank copula has to be unequal to 0.")
-  	if(Model1.family[i]==7 && Model1.par[i]<=0) stop("The first parameter of the BB1 copula has to be positive.")
-  	if(Model1.family[i]==7 && Model1.par2[i]<1) stop("The second parameter of the BB1 copula has to be in the interval [1,oo).")
-  	if(Model1.family[i]==9 && Model1.par[i]<1) stop("The first parameter of the BB7 copula has to be in the interval [1,oo).")
-  	if(Model1.family[i]==9 && Model1.par2[i]<=0) stop("The second parameter of the BB7 copula has to be positive.")
+  	if((Model1.family[i]==7 || Model1.family[i]==17) && Model1.par[i]<=0) stop("The first parameter of the BB1 copula has to be positive.")
+  	if((Model1.family[i]==7 || Model1.family[i]==17) && Model1.par2[i]<1) stop("The second parameter of the BB1 copula has to be in the interval [1,oo).")
+	if((Model1.family[i]==8 || Model1.family[i]==18) && Model1.par[i]<=0) stop("The first parameter of the BB6 copula has to be in the interval [1,oo).")
+	if((Model1.family[i]==8 || Model1.family[i]==18) && Model1.par2[i]<1) stop("The second parameter of the BB6 copula has to be in the interval [1,oo).")
+  	if((Model1.family[i]==9 || Model1.family[i]==19) && Model1.par[i]<1) stop("The first parameter of the BB7 copula has to be in the interval [1,oo).")
+  	if((Model1.family[i]==9 || Model1.family[i]==19) && Model1.par2[i]<=0) stop("The second parameter of the BB7 copula has to be positive.")
+	if((Model1.family[i]==10 || Model1.family[i]==20) && Model1.par[i]<1) stop("The first parameter of the BB8 copula has to be in the interval [1,oo).")
+	if((Model1.family[i]==10 || Model1.family[i]==20) && (Model1.par2[i]<=0 || Model1.par2[i]>1)) stop("The second parameter of the BB8 copula has to be in the interval (0,1].") 
   	if((Model1.family[i]==23 || Model1.family[i]==33) && Model1.par[i]>=0) stop("The parameter of the rotated Clayton copula has to be negative.")
   	if((Model1.family[i]==24 || Model1.family[i]==34) && Model1.par[i]>-1) stop("The parameter of the rotated Gumbel copula has to be in the interval (-oo,-1].")
   	if((Model1.family[i]==26 || Model1.family[i]==36) && Model1.par[i]>=-1) stop("The parameter of the rotated Joe copula has to be in the interval (-oo,-1).")
+	if((Model1.family[i]==27 || Model1.family[i]==37) && Model1.par[i]>=0) stop("The first parameter of the rotated BB1 copula has to be negative.")
+	if((Model1.family[i]==27 || Model1.family[i]==37) && Model1.par2[i]>-1) stop("The second parameter of the rotated BB1 copula has to be in the interval (-oo,-1].")
+	if((Model1.family[i]==28 || Model1.family[i]==38) && Model1.par[i]>=0) stop("The first parameter of the rotated BB6 copula has to be in the interval (-oo,-1].")
+	if((Model1.family[i]==28 || Model1.family[i]==38) && Model1.par2[i]>-1) stop("The second parameter of the rotated BB6 copula has to be in the interval (-oo,-1].")
+	if((Model1.family[i]==29 || Model1.family[i]==39) && Model1.par[i]>-1) stop("The first parameter of the rotated BB7 copula has to be in the interval (-oo,-1].")
+	if((Model1.family[i]==29 || Model1.family[i]==39) && Model1.par2[i]>=0) stop("The second parameter of the rotated BB7 copula has to be negative.")
+	if((Model1.family[i]==30 || Model1.family[i]==40) && Model1.par[i]>-1) stop("The first parameter of the rotated BB8 copula has to be in the interval (-oo,-1].")
+	if((Model1.family[i]==30 || Model1.family[i]==40) && (Model1.par2[i]>=0 || Model1.par2[i]<(-1))) stop("The second parameter of the rotated BB8 copula has to be in the interval [-1,0).")
 	}
 for(i in 1:(d*(d-1)/2))
 	{
-	if(!(Model2.family[i] %in% c(0,1,2,3,4,5,6,7,9,13,14,16,23,24,26,33,34,36))) stop("Copula family not implemented. (in Model1.family)")
+	if(!(Model2.family[i] %in% c(0,1:10,13,14,16:20,23,24,26:30,33,34,36:40))) stop("Copula family not implemented (in Model1.family).")
   	if((Model2.family[i]==1 || Model2.family[i]==2) && abs(Model2.par[i])>=1) stop("The parameter of the Gaussian and t-copula has to be in the interval (-1,1).")
 	if(Model2.family[i]==2 && Model2.par2[i]<=1) stop("The degrees of freedom parameter of the t-copula has to be larger than 1.")
   	if((Model2.family[i]==3 || Model2.family[i]==13) && Model2.par[i]<=0) stop("The parameter of the Clayton copula has to be positive.")
   	if((Model2.family[i]==4 || Model2.family[i]==14) && Model2.par[i]<1) stop("The parameter of the Gumbel copula has to be in the interval [1,oo).")
   	if((Model2.family[i]==6 || Model2.family[i]==16) && Model2.par[i]<=1) stop("The parameter of the Joe copula has to be in the interval (1,oo).")	
   	if(Model2.family[i]==5 && Model2.par[i]==0) stop("The parameter of the Frank copula has to be unequal to 0.")
-  	if(Model2.family[i]==7 && Model2.par[i]<=0) stop("The first parameter of the BB1 copula has to be positive.")
-  	if(Model2.family[i]==7 && Model2.par2[i]<1) stop("The second parameter of the BB1 copula has to be in the interval [1,oo).")
-  	if(Model2.family[i]==9 && Model2.par[i]<1) stop("The first parameter of the BB7 copula has to be in the interval [1,oo).")
-  	if(Model2.family[i]==9 && Model2.par2[i]<=0) stop("The second parameter of the BB7 copula has to be positive.")
+  	if((Model2.family[i]==7 || Model2.family[i]==17) && Model2.par[i]<=0) stop("The first parameter of the BB1 copula has to be positive.")
+  	if((Model2.family[i]==7 || Model2.family[i]==17) && Model2.par2[i]<1) stop("The second parameter of the BB1 copula has to be in the interval [1,oo).")
+	if((Model2.family[i]==8 || Model2.family[i]==18) && Model2.par[i]<=0) stop("The first parameter of the BB6 copula has to be in the interval [1,oo).")
+	if((Model2.family[i]==8 || Model2.family[i]==18) && Model2.par2[i]<1) stop("The second parameter of the BB6 copula has to be in the interval [1,oo).")
+  	if((Model2.family[i]==9 || Model2.family[i]==19) && Model2.par[i]<1) stop("The first parameter of the BB7 copula has to be in the interval [1,oo).")
+  	if((Model2.family[i]==9 || Model2.family[i]==19) && Model2.par2[i]<=0) stop("The second parameter of the BB7 copula has to be positive.")
+	if((Model2.family[i]==10 || Model2.family[i]==20) && Model2.par[i]<1) stop("The first parameter of the BB8 copula has to be in the interval [1,oo).")
+	if((Model2.family[i]==10 || Model2.family[i]==20) && (Model2.par2[i]<=0 || Model2.par2[i]>1)) stop("The second parameter of the BB8 copula has to be in the interval (0,1].") 
   	if((Model2.family[i]==23 || Model2.family[i]==33) && Model2.par[i]>=0) stop("The parameter of the rotated Clayton copula has to be negative.")
   	if((Model2.family[i]==24 || Model2.family[i]==34) && Model2.par[i]>-1) stop("The parameter of the rotated Gumbel copula has to be in the interval (-oo,-1].")
   	if((Model2.family[i]==26 || Model2.family[i]==36) && Model2.par[i]>=-1) stop("The parameter of the rotated Joe copula has to be in the interval (-oo,-1).")
+	if((Model2.family[i]==27 || Model2.family[i]==37) && Model2.par[i]>=0) stop("The first parameter of the rotated BB1 copula has to be negative.")
+	if((Model2.family[i]==27 || Model2.family[i]==37) && Model2.par2[i]>-1) stop("The second parameter of the rotated BB1 copula has to be in the interval (-oo,-1].")
+	if((Model2.family[i]==28 || Model2.family[i]==38) && Model2.par[i]>=0) stop("The first parameter of the rotated BB6 copula has to be in the interval (-oo,-1].")
+	if((Model2.family[i]==28 || Model2.family[i]==38) && Model2.par2[i]>-1) stop("The second parameter of the rotated BB6 copula has to be in the interval (-oo,-1].")
+	if((Model2.family[i]==29 || Model2.family[i]==39) && Model2.par[i]>-1) stop("The first parameter of the rotated BB7 copula has to be in the interval (-oo,-1].")
+	if((Model2.family[i]==29 || Model2.family[i]==39) && Model2.par2[i]>=0) stop("The second parameter of the rotated BB7 copula has to be negative.")
+	if((Model2.family[i]==30 || Model2.family[i]==40) && Model2.par[i]>-1) stop("The first parameter of the rotated BB8 copula has to be in the interval (-oo,-1].")
+	if((Model2.family[i]==30 || Model2.family[i]==40) && (Model2.par2[i]>=0 || Model2.par2[i]<(-1))) stop("The second parameter of the rotated BB8 copula has to be in the interval [-1,0).")
 	}
 
 
@@ -80,8 +104,8 @@ Model2.ll[i] = LogLikelihood(Model2.family, data[i,Model2.order], Model2.par, Mo
 # model1.ll<-sapply(data,function (x) LogLikelihood(Model1.family, x, Model1.par, Model1.par2, Model1.type))
  
 
-anz.1 = d*(d-1)/2 + sum(Model1.family == 2,na.rm=TRUE) + sum(Model1.family == 7,na.rm=TRUE) + sum(Model1.family == 8,na.rm=TRUE) + sum(Model2.family == 9,na.rm=TRUE)
-anz.2 = d*(d-1)/2 + sum(Model2.family == 2,na.rm=TRUE) + sum(Model2.family == 7,na.rm=TRUE) + sum(Model2.family == 8,na.rm=TRUE) + sum(Model2.family == 9,na.rm=TRUE)
+  anz.1 = sum(Model1.family >= 1, na.rm=TRUE) + sum(Model1.family %in% c(2,7:10,17:20,27:30,37:40),na.rm=TRUE)	
+  anz.2 = sum(Model2.family >= 1, na.rm=TRUE) + sum(Model2.family %in% c(2,7:10,17:20,27:30,37:40),na.rm=TRUE)
 
 B = sum(Model1.ll-Model2.ll > 0)
   B.Schwarz = sum(Model1.ll-Model2.ll-(anz.1-anz.2)*log(N)/(2*N) > 0)
