@@ -22,21 +22,7 @@
 void archCDF(double* u, double* v, int* n, double* param, int* copula, double* out)
 {
 	int j;
-	double *t1, *t2, *t3, *t4, *t5, *t6, *t7, *t8, *t9, *t10, *t11, *t12, *t13, *t14;
-	t1 = Calloc(*n,double);
-	t2 = Calloc(*n,double);
-	t3 = Calloc(*n,double);
-	t4 = Calloc(*n,double);
-	t5 = Calloc(*n,double);
-	t6 = Calloc(*n,double);
-	t7 = Calloc(*n,double);
-	t8 = Calloc(*n,double);
-	t9 = Calloc(*n,double);
-	t10 = Calloc(*n,double);
-	t11 = Calloc(*n,double);
-	t12 = Calloc(*n,double);
-	t13 = Calloc(*n,double);
-	t14 = Calloc(*n,double);
+	double t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, t12, t13, t14;
 
 	for(j=0;j<*n;j++)
 	{
@@ -48,190 +34,161 @@ void archCDF(double* u, double* v, int* n, double* param, int* copula, double* o
 		{
 		if(*copula==3)	//Clayton
 		{
-			t1[j] = pow(u[j],-param[0]);
-			t2[j] = pow(v[j],-param[0]);
-			t3[j] = t1[j]+t2[j]-1;
-			out[j] = pow(t3[j],-1/param[0]);
+			t1 = pow(u[j],-param[0]);
+			t2 = pow(v[j],-param[0]);
+			t3 = t1+t2-1;
+			out[j] = pow(t3,-1/param[0]);
 		}
 		else if(*copula==4)	//Gumbel
 		{
-			t1[j] = -log(u[j]);
-			t2[j] = -log(v[j]);
-			t3[j] = pow(t1[j],param[0]);
-			t4[j] = pow(t2[j],param[0]);
-			t5[j] = t3[j]+t4[j];
-			t6[j] = -pow(t5[j],1/param[0]);
-			out[j] = exp(t6[j]);
+			t1 = -log(u[j]);
+			t2 = -log(v[j]);
+			t3 = pow(t1,param[0]);
+			t4 = pow(t2,param[0]);
+			t5 = t3+t4;
+			t6 = -pow(t5,1/param[0]);
+			out[j] = exp(t6);
 		}
 		else if(*copula==5)	//Frank
 		{
 			double nu;
-			t1[j] = -param[0]*u[j];
-			t2[j] = -param[0]*v[j];
-			t3[j] = exp(t1[j]);
-			t4[j] = exp(t2[j]);
-			t5[j] = 1-t3[j];
-			t6[j] = 1-t4[j];
+			t1 = -param[0]*u[j];
+			t2 = -param[0]*v[j];
+			t3 = exp(t1);
+			t4 = exp(t2);
+			t5 = 1-t3;
+			t6 = 1-t4;
 			nu = 1-exp(-param[0]);
-			t7[j] = t5[j]*t6[j];
-			t8[j] = nu-t7[j];
-			out[j] = -1/param[0]*log(t8[j]/nu);
+			t7 = t5*t6;
+			t8 = nu-t7;
+			out[j] = -1/param[0]*log(t8/nu);
 		}
 		else if(*copula==6)	//Joe
 		{
-			t1[j] = 1-u[j];
-			t2[j] = 1-v[j];
-			t3[j] = pow(t1[j],param[0]);
-			t4[j] = pow(t2[j],param[0]);
-			t5[j] = t3[j]*t4[j];
-			out[j] = 1-pow(t3[j]+t4[j]-t5[j],1/param[0]);
+			t1 = 1-u[j];
+			t2 = 1-v[j];
+			t3 = pow(t1,param[0]);
+			t4 = pow(t2,param[0]);
+			t5 = t3*t4;
+			out[j] = 1-pow(t3+t4-t5,1/param[0]);
 		}
 		else if(*copula==7)	//BB1
 		{
-			t1[j] = pow(u[j],-param[0]);
-			t2[j] = pow(v[j],-param[0]);
-			t3[j] = t1[j]-1;
-			t4[j] = t2[j]-1;
-			t5[j] = pow(t3[j],param[1]);
-			t6[j] = pow(t4[j],param[1]);
-			t7[j] = t5[j]+t6[j];
-			t8[j] = pow(t7[j],1/param[1]);
-			out[j] = pow(1+t8[j],-1/param[0]);
+			t1 = pow(u[j],-param[0]);
+			t2 = pow(v[j],-param[0]);
+			t3 = t1-1;
+			t4 = t2-1;
+			t5 = pow(t3,param[1]);
+			t6 = pow(t4,param[1]);
+			t7 = t5+t6;
+			t8 = pow(t7,1/param[1]);
+			out[j] = pow(1+t8,-1/param[0]);
 		}
 		else if(*copula==8)	//BB6
 		{
-			t1[j] = 1-u[j];
-			t2[j] = 1-v[j];
-			t3[j] = pow(t1[j],param[0]);
-			t4[j] = pow(t2[j],param[0]);
-			t5[j] = 1-t3[j];
-			t6[j] = 1-t4[j];
-			t7[j] = -log(t5[j]);
-			t8[j] = -log(t6[j]);
-			t9[j] = pow(t7[j],param[1]);
-			t10[j] = pow(t8[j],param[1]);
-			t11[j] = t9[j]+t10[j];
-			t12[j] = pow(t11[j],1/param[1]);
-			t13[j] = exp(-t12[j]);
-			t14[j] = 1-t13[j];
-			out[j] = 1-pow(t14[j],1/param[0]);
+			t1 = 1-u[j];
+			t2 = 1-v[j];
+			t3 = pow(t1,param[0]);
+			t4 = pow(t2,param[0]);
+			t5 = 1-t3;
+			t6 = 1-t4;
+			t7 = -log(t5);
+			t8 = -log(t6);
+			t9 = pow(t7,param[1]);
+			t10 = pow(t8,param[1]);
+			t11 = t9+t10;
+			t12 = pow(t11,1/param[1]);
+			t13 = exp(-t12);
+			t14 = 1-t13;
+			out[j] = 1-pow(t14,1/param[0]);
 		}
 		else if(*copula==9)	//BB7
 		{
-			t1[j] = 1-u[j];
-			t2[j] = 1-v[j];
-			t3[j] = pow(t1[j],param[0]);
-			t4[j] = pow(t2[j],param[0]);
-			t5[j] = 1-t3[j];
-			t6[j] = 1-t4[j];
-			t7[j] = pow(t5[j],-param[1]);
-			t8[j] = pow(t6[j],-param[1]);
-			t9[j] = t7[j]+t8[j]-1;
-			t10[j] = pow(t9[j],-1/param[1]);
-			t11[j] = 1-t10[j];
-			t12[j] = pow(t11[j],1/param[0]);
-			out[j] = 1-t12[j];
+			t1 = 1-u[j];
+			t2 = 1-v[j];
+			t3 = pow(t1,param[0]);
+			t4 = pow(t2,param[0]);
+			t5 = 1-t3;
+			t6 = 1-t4;
+			t7 = pow(t5,-param[1]);
+			t8 = pow(t6,-param[1]);
+			t9 = t7+t8-1;
+			t10 = pow(t9,-1/param[1]);
+			t11 = 1-t10;
+			t12 = pow(t11,1/param[0]);
+			out[j] = 1-t12;
 		}
 		else if(*copula==10)    //BB8
         {
             double nu;
-            t1[j] = param[1]*u[j];
-            t2[j] = param[1]*v[j];
-            t3[j] = 1-t1[j];
-            t4[j] = 1-t2[j];
-            t5[j] = pow(t3[j],param[0]);
-            t6[j] = pow(t4[j],param[0]);
-            t7[j] = 1-t5[j];
-            t8[j] = 1-t6[j];
+            t1 = param[1]*u[j];
+            t2 = param[1]*v[j];
+            t3 = 1-t1;
+            t4 = 1-t2;
+            t5 = pow(t3,param[0]);
+            t6 = pow(t4,param[0]);
+            t7 = 1-t5;
+            t8 = 1-t6;
             nu = 1-param[1];
             nu = pow(nu,param[0]);
             nu = 1-nu;
 			nu = 1/nu;
-            t9[j] = 1-nu*t7[j]*t8[j];
-            t10[j] = pow(t9[j],1/param[0]);
-            out[j] = 1/param[1]*(1-t10[j]);
+            t9 = 1-nu*t7*t8;
+            t10 = pow(t9,1/param[0]);
+            out[j] = 1/param[1]*(1-t10);
         }
 
 		}
 	}
-	Free(t1); Free(t2); Free(t3); Free(t4); Free(t5); Free(t6); Free(t7); Free(t8);
-	Free(t9); Free(t10); Free(t11); Free(t12); Free(t13); Free(t14);
+
 }
 
 
 
+
+
 void dbb1(double* u, double* v, int* n, double* param, double* out)
 {
 	int i;
 	double th, de;
-	double *t1, *t2, *t3, *t16, *t17, *t38, *t39, *t4, *t5, *t6, *t7, *t8,*t9, *t10, *t12, *t13, *t20, *t24, *t25, *t27, *t29, *t32, *t33, *t34, *t36, *t43, *t59;
-	t1 = Calloc(*n,double);
-	t2 = Calloc(*n,double);
-	t3 = Calloc(*n,double);
-	t16 = Calloc(*n,double);
-	t17 = Calloc(*n,double);
-	t38 = Calloc(*n,double);
-	t39 = Calloc(*n,double);
-	t4 = Calloc(*n,double);
-	t5 = Calloc(*n,double);
-	t6 = Calloc(*n,double);
-	t7 = Calloc(*n,double);
-	t8 = Calloc(*n,double);
-	t9 = Calloc(*n,double);
-	t10 = Calloc(*n,double);
-	t12 = Calloc(*n,double);
-	t13 = Calloc(*n,double);
-	t20 = Calloc(*n,double);
-	t24 = Calloc(*n,double);
-	t25 = Calloc(*n,double);
-	t27 = Calloc(*n,double);
-	t29 = Calloc(*n,double);
-	t32 = Calloc(*n,double);
-	t33 = Calloc(*n,double);
-	t34 = Calloc(*n,double);
-	t36 = Calloc(*n,double);
-	t43 = Calloc(*n,double);
-	t59 = Calloc(*n,double);
+	double t1, t2, t3, t16, t17, t38, t39, t4, t5, t6, t7, t9, t10, t12, t13, t20, t24, t25, t27, t29, t32, t33, t34, t36, t43, t59;
 
 	th = param[0];
 	de = param[1];
 
 	for(i=0;i<*n;i++)
 	{
-		t1[i] = pow(u[i],(-th));
-		t2[i] = t1[i]-1.0;
-		t3[i] = pow(t2[i],de);
-		t16[i] = 1./u[i];
-		t17[i] = 1./t2[i];
-		t38[i] = t1[i]*t16[i];
-		t39[i] = t38[i]*t17[i];
-		t4[i] = pow(v[i],(-th));
-		t5[i] = t4[i]-1.0;
-		t6[i] = pow(t5[i],de);
-		t7[i] = t3[i]+t6[i];
-		t9[i] = pow(t7[i],(1./de));
-		t10[i] = 1.0+t9[i];
-		t12[i] = pow(t10[i],(-1./th));
-		t13[i] = t12[i]*t9[i];
-		t20[i] = 1./t10[i];
-		t24[i] = t9[i]*t9[i];
-		t25[i] = t12[i]*t24[i];
-		t27[i] = 1./v[i];
-		t29[i] = 1./t5[i];
-		t32[i] = t7[i]*t7[i];
-		t33[i] = 1./t32[i];
-		t34[i] = t10[i]*t10[i];
-		t36[i] = t33[i]/t34[i];
-		t43[i] = t4[i]*th;
-		t59[i] = t43[i]*t27[i]*t29[i];
+		t1 = pow(u[i],(-th));
+		t2 = t1-1.0;
+		t3 = pow(t2,de);
+		t16 = 1./u[i];
+		t17 = 1./t2;
+		t38 = t1*t16;
+		t39 = t38*t17;
+		t4 = pow(v[i],(-th));
+		t5 = t4-1.0;
+		t6 = pow(t5,de);
+		t7 = t3+t6;
+		t9 = pow(t7,(1./de));
+		t10 = 1.0+t9;
+		t12 = pow(t10,(-1./th));
+		t13 = t12*t9;
+		t20 = 1./t10;
+		t24 = t9*t9;
+		t25 = t12*t24;
+		t27 = 1./v[i];
+		t29 = 1./t5;
+		t32 = t7*t7;
+		t33 = 1./t32;
+		t34 = t10*t10;
+		t36 = t33/t34;
+		t43 = t4*th;
+		t59 = t43*t27*t29;
 
-		out[i] = t25[i]*t6[i]*t27[i]*t4[i]*t29[i]*t36[i]*t3[i]*t39[i]-t13[i]*t6[i]*t43[i]*t27[i]*t29[i]*t33[i]*t3[i]*t38[i]*t17[i]*t20[i]+
-        t13[i]*t3[i]*t38[i]*t17[i]*t33[i]*t20[i]*t6[i]*de*t59[i]+t25[i]*t3[i]*t39[i]*t36[i]*t6[i]*t59[i];
+		out[i] = t25*t6*t27*t4*t29*t36*t3*t39-t13*t6*t43*t27*t29*t33*t3*t38*t17*t20+
+        t13*t3*t38*t17*t33*t20*t6*de*t59+t25*t3*t39*t36*t6*t59;
 	}
 
-	Free(t1);Free(t2);Free(t3);Free(t16);Free(t17);Free(t38);Free(t39);Free(t4);Free(t5);Free(t6);
-	Free(t7);Free(t8);Free(t9);Free(t10);Free(t12);Free(t13);Free(t20);Free(t24);Free(t25);Free(t27);
-	Free(t29);Free(t32);Free(t33);Free(t34);Free(t36);Free(t43);Free(t59);
 }
 
 
@@ -239,99 +196,57 @@ void dbb6(double* u, double* v, int* n, double* param, double* out)
 {
 	int i;
 	double th, de;
-	double *t1, *t2, *t3, *t4, *t5, *t12, *t16, *t32, *t38, *t39, *t40, *t47, *t50, *t61, *t90, *t6, *t7, *t8,*t9, *t10, *t11, *t13, *t14, *t15, *t17, *t35, *t36, *t37, *t42, *t48, *t53, *t56, *t57, *t59, *t78, *t80, *t87, *t93;
-	t1 = Calloc(*n,double);
-	t2 = Calloc(*n,double);
-	t3 = Calloc(*n,double);
-	t4 = Calloc(*n,double);
-	t5 = Calloc(*n,double);
-	t12 = Calloc(*n,double);
-	t16 = Calloc(*n,double);
-	t32 = Calloc(*n,double);
-	t38 = Calloc(*n,double);
-	t39 = Calloc(*n,double);
-	t40 = Calloc(*n,double);
-	t47 = Calloc(*n,double);
-	t50 = Calloc(*n,double);
-	t61 = Calloc(*n,double);
-	t90 = Calloc(*n,double);
-	t6 = Calloc(*n,double);
-	t7 = Calloc(*n,double);
-	t8 = Calloc(*n,double);
-	t9 = Calloc(*n,double);
-	t10 = Calloc(*n,double);
-	t11 = Calloc(*n,double);
-	t13 = Calloc(*n,double);
-	t14 = Calloc(*n,double);
-	t15 = Calloc(*n,double);
-	t17 = Calloc(*n,double);
-	t35 = Calloc(*n,double);
-	t36 = Calloc(*n,double);
-	t37 = Calloc(*n,double);
-	t42 = Calloc(*n,double);
-	t48 = Calloc(*n,double);
-	t53 = Calloc(*n,double);
-	t56 = Calloc(*n,double);
-	t57 = Calloc(*n,double);
-	t59 = Calloc(*n,double);
-	t78 = Calloc(*n,double);
-	t80 = Calloc(*n,double);
-	t87 = Calloc(*n,double);
-	t93 = Calloc(*n,double);
+	double t1, t2, t3, t4, t5, t12, t16, t32, t38, t39, t40, t47, t50, t61, t90, t6, t7, t8, t9, t10, t11, t13, t14, t15, t17, t35, t36, t37, t42, t48, t53, t56, t57, t59, t78, t80, t87, t93;
 
 	th = param[0];
 	de = param[1];
 
 	for(i=0;i<*n;i++)
 	{
-		  t1[i] = 1.0-u[i];
-		  t2[i] = pow(t1[i],th);
-		  t3[i] = 1.0-t2[i];
-		  t4[i] = log(t3[i]);
-		  t5[i] = pow(-t4[i],de);
-		  t12[i] = 1/de;
-		  t16[i] = 1/th;
-		  t32[i] = de-1.0;
-		  t38[i] = 2.0*de;
-		  t39[i] = -1.0+t38[i];
-		  t40[i] = pow(-t4[i],t39[i]);
-		  t47[i] = 3.0*de-1.0;
-		  t50[i] = pow(-t4[i],t32[i]);
-		  t61[i] = pow(-t4[i],t47[i]);
-		  t90[i] = pow(-t4[i],t38[i]);
-		  t6[i] = 1.0-v[i];
-		  t7[i] = pow(t6[i],th);
-		  t8[i] = 1.0-t7[i];
-		  t9[i] = log(t8[i]);
-		  t10[i] = pow(-t9[i],de);
-		  t11[i] = t5[i]+t10[i];
-		  t13[i] = pow(t11[i],t12[i]);
-		  t14[i] = exp(-t13[i]);
-		  t15[i] = 1.0-t14[i];
-		  t17[i] = pow(t15[i],t16[i]);
-		  t35[i] = pow(t11[i],-2.0*t32[i]*t12[i]);
-		  t36[i] = t35[i]*th;
-		  t37[i] = exp(t13[i]);
-		  t42[i] = pow(-t9[i],t39[i]);
-		  t48[i] = pow(-t9[i],t47[i]);
-		  t53[i] = t13[i]*de;
-		  t56[i] = pow(-t9[i],t32[i]);
-		  t57[i] = t37[i]*t50[i]*t56[i];
-		  t59[i] = t13[i]*th;
-		  t78[i] = t37[i]-1.0;
-		  t80[i] = pow(t78[i]*t14[i],t16[i]);
-		  t87[i] = t78[i]*t78[i];
-		  t93[i] = pow(-t9[i],t38[i]);
+		  t1 = 1.0-u[i];
+		  t2 = pow(t1,th);
+		  t3 = 1.0-t2;
+		  t4 = log(t3);
+		  t5 = pow(-t4,de);
+		  t12 = 1/de;
+		  t16 = 1/th;
+		  t32 = de-1.0;
+		  t38 = 2.0*de;
+		  t39 = -1.0+t38;
+		  t40 = pow(-t4,t39);
+		  t47 = 3.0*de-1.0;
+		  t50 = pow(-t4,t32);
+		  t61 = pow(-t4,t47);
+		  t90 = pow(-t4,t38);
+		  t6 = 1.0-v[i];
+		  t7 = pow(t6,th);
+		  t8 = 1.0-t7;
+		  t9 = log(t8);
+		  t10 = pow(-t9,de);
+		  t11 = t5+t10;
+		  t13 = pow(t11,t12);
+		  t14 = exp(-t13);
+		  t15 = 1.0-t14;
+		  t17 = pow(t15,t16);
+		  t35 = pow(t11,-2.0*t32*t12);
+		  t36 = t35*th;
+		  t37 = exp(t13);
+		  t42 = pow(-t9,t39);
+		  t48 = pow(-t9,t47);
+		  t53 = t13*de;
+		  t56 = pow(-t9,t32);
+		  t57 = t37*t50*t56;
+		  t59 = t13*th;
+		  t78 = t37-1.0;
+		  t80 = pow(t78*t14,t16);
+		  t87 = t78*t78;
+		  t93 = pow(-t9,t38);
 
-		  out[i] = (2.0*t36[i]*t37[i]*t40[i]*t42[i]+t36[i]*t37[i]*t48[i]*t50[i]+t53[i]*th*t57[i]-t59[i]*t57[i]+
-					t36[i]* t37[i]*t61[i]*t56[i]-2.0*t35[i]*t40[i]*t42[i]-t35[i]*t61[i]*t56[i]-t53[i]*th*t50[i]*t56[i]+t59[i]*t50[i]*t56[i]-
-					t35[i]*t48[i]*t50[i]) *t80[i]*t7[i]*t2[i]/t3[i]/t8[i]/t87[i]/(t90[i]+2.0*t5[i]*t10[i]+t93[i])/t1[i]/t6[i];
+		  out[i] = (2.0*t36*t37*t40*t42+t36*t37*t48*t50+t53*th*t57-t59*t57+
+					t36*t37*t61*t56-2.0*t35*t40*t42-t35*t61*t56-t53*th*t50*t56+t59*t50*t56-
+					t35*t48*t50) *t80*t7*t2/t3/t8/t87/(t90+2.0*t5*t10+t93)/t1/t6;
 	}
 
-	Free(t1);Free(t2);Free(t3);Free(t4);Free(t5);Free(t12);Free(t16);Free(t32);Free(t38);Free(t39);
-	Free(t40);Free(t47);Free(t50);Free(t61);Free(t90);Free(t6);Free(t7);Free(t8);Free(t9);Free(t10);
-	Free(t11);Free(t13);Free(t14);Free(t15);Free(t17);Free(t35);Free(t36);Free(t37);Free(t42);Free(t48);
-	Free(t53);Free(t56);Free(t57);Free(t59);Free(t78);Free(t80);Free(t87);Free(t93);
 }
 
 
@@ -339,75 +254,45 @@ void dbb7(double* u, double* v, int* n, double* param, double* out)
 {
 	int i;
 	double th, de;
-	double *t1, *t2, *t3, *t4, *t5, *t6, *t7, *t8, *t9, *t11, *t12, *t14, *t15, *t16, *t18, *t20, *t23, *t24, *t25, *t27, *t30, *t31, *t32, *t35, *t37, *t42, *t54;
-	t1 = Calloc(*n,double);
-	t2 = Calloc(*n,double);
-	t3 = Calloc(*n,double);
-	t4 = Calloc(*n,double);
-	t5 = Calloc(*n,double);
-	t6 = Calloc(*n,double);
-	t7 = Calloc(*n,double);
-	t8 = Calloc(*n,double);
-	t9 = Calloc(*n,double);
-	t11 = Calloc(*n,double);
-	t12 = Calloc(*n,double);
-	t14 = Calloc(*n,double);
-	t15 = Calloc(*n,double);
-	t16 = Calloc(*n,double);
-	t18 = Calloc(*n,double);
-	t20 = Calloc(*n,double);
-	t23 = Calloc(*n,double);
-	t24 = Calloc(*n,double);
-	t25 = Calloc(*n,double);
-	t27 = Calloc(*n,double);
-	t30 = Calloc(*n,double);
-	t31 = Calloc(*n,double);
-	t32 = Calloc(*n,double);
-	t35 = Calloc(*n,double);
-	t37 = Calloc(*n,double);
-	t42 = Calloc(*n,double);
-	t54 = Calloc(*n,double);
+	double t1, t2, t3, t4, t5, t6, t7, t8, t9, t11, t12, t14, t15, t16, t18, t20, t23, t24, t25, t27, t30, t31, t32, t35, t37, t42, t54;
 
 	th = param[0];
 	de = param[1];
 
 	for(i=0;i<*n;i++)
 	{
-		t1[i] = 1.0-u[i];
-		t2[i] = pow(t1[i],th);
-		t3[i] = 1.0-t2[i];
-		t4[i] = pow(t3[i],-de);
-		t5[i] = 1.0-v[i];
-		t6[i] = pow(t5[i],th);
-		t7[i] = 1.0-t6[i];
-		t8[i] = pow(t7[i],-de);
-		t9[i] = t4[i]+t8[i]-1.0;
-		t11[i] = pow(t9[i],-1.0/de);
-		t12[i] = 1.0-t11[i];
-		t14[i] = pow(t12[i],1.0/th);
-		t15[i] = t11[i]*t11[i];
-		t16[i] = t14[i]*t15[i];
-		t18[i] = 1./t5[i];
-		t20[i] = 1./t7[i];
-		t23[i] = t9[i]*t9[i];
-		t24[i] = 1./t23[i];
-		t25[i] = t12[i]*t12[i];
-		t27[i] = t24[i]/t25[i];
-		t30[i] = t2[i]/t1[i];
-		t31[i] = 1./t3[i];
-		t32[i] = t30[i]*t31[i];
-		t35[i] = t14[i]*t11[i];
-		t37[i] = t6[i]*th;
-		t42[i] = 1./t12[i];
-		t54[i] = t37[i]*t18[i]*t20[i];
+		t1 = 1.0-u[i];
+		t2 = pow(t1,th);
+		t3 = 1.0-t2;
+		t4 = pow(t3,-de);
+		t5 = 1.0-v[i];
+		t6 = pow(t5,th);
+		t7 = 1.0-t6;
+		t8 = pow(t7,-de);
+		t9 = t4+t8-1.0;
+		t11 = pow(t9,-1.0/de);
+		t12 = 1.0-t11;
+		t14 = pow(t12,1.0/th);
+		t15 = t11*t11;
+		t16 = t14*t15;
+		t18 = 1./t5;
+		t20 = 1./t7;
+		t23 = t9*t9;
+		t24 = 1./t23;
+		t25 = t12*t12;
+		t27 = t24/t25;
+		t30 = t2/t1;
+		t31 = 1./t3;
+		t32 = t30*t31;
+		t35 = t14*t11;
+		t37 = t6*th;
+		t42 = 1./t12;
+		t54 = t37*t18*t20;
 
-		out[i] = -t16[i]*t8[i]*t6[i]*t18[i]*t20[i]*t27[i]*t4[i]*t32[i] + t35[i]*t8[i]*t37[i]*t18[i]*t20[i]*t24[i]*t4[i]*t30[i]*t31[i]*t42[i]+
-   t35[i]*t4[i]*t30[i]*t31[i]*t24[i]*t42[i]*t8[i]*de*t54[i]+t16[i]*t4[i]*t32[i]*t27[i]*t8[i]*t54[i];
+		out[i] = -t16*t8*t6*t18*t20*t27*t4*t32 + t35*t8*t37*t18*t20*t24*t4*t30*t31*t42+
+   t35*t4*t30*t31*t24*t42*t8*de*t54+t16*t4*t32*t27*t8*t54;
 	}
 
-	Free(t1);Free(t2);Free(t3);Free(t4);Free(t5);Free(t6);Free(t7);Free(t8);Free(t9);
-	Free(t11);Free(t12);Free(t14);Free(t15);Free(t16);Free(t18);Free(t20);Free(t23);Free(t24);Free(t25);
-	Free(t27);Free(t30);Free(t31);Free(t32);Free(t35);Free(t37);Free(t42);Free(t54);
 }
 
 
@@ -415,81 +300,46 @@ void dbb8(double* u, double* v, int* n, double* param, double* out)
 {
 	int i;
 	double th, de;
-	double *t2, *t3, *t4, *t12, *t16, *t6, *t7, *t8, *t10, *t11, *t13, *t15, *t17, *t33, *t38, *t39, *t49, *t59, *t69, *t25, *t26, *t29, *t44, *t45, *t50, *t54, *t62, *t67;
-	t2 = Calloc(*n,double);
-	t3 = Calloc(*n,double);
-	t4 = Calloc(*n,double);
-	t12 = Calloc(*n,double);
-	t16 = Calloc(*n,double);
-	t6 = Calloc(*n,double);
-	t7 = Calloc(*n,double);
-	t8 = Calloc(*n,double);
-	t10 = Calloc(*n,double);
-	t11 = Calloc(*n,double);
-	t13 = Calloc(*n,double);
-	t15 = Calloc(*n,double);
-	t17 = Calloc(*n,double);
-	t33 = Calloc(*n,double);
-	t38 = Calloc(*n,double);
-	t39 = Calloc(*n,double);
-	t49 = Calloc(*n,double);
-	t59 = Calloc(*n,double);
-	t69 = Calloc(*n,double);
-	t25 = Calloc(*n,double);
-	t26 = Calloc(*n,double);
-	t29 = Calloc(*n,double);
-	t44 = Calloc(*n,double);
-	t45 = Calloc(*n,double);
-	t50 = Calloc(*n,double);
-	t54 = Calloc(*n,double);
-	t62 = Calloc(*n,double);
-	t67 = Calloc(*n,double);
+	double t2, t3, t4, t12, t16, t6, t7, t8, t10, t11, t13, t15, t17, t33, t38, t39, t49, t59, t69, t25, t26, t29, t44, t45, t50, t54, t62, t67;
 
 	th = param[0];
 	de = param[1];
 
 	for(i=0;i<*n;i++)
 	{
-		  t2[i] = 1.0-de*u[i];
-		  t3[i] = pow(t2[i],th);
-		  t4[i] = 1.0-t3[i];
-		  t10[i] = 1.0-de;
-		  t11[i] = pow(t10[i],th);
-		  t12[i] = 1.0-t11[i];
-		  t13[i] = 1/t12[i];
-		  t16[i] = 1/th;
-		  t33[i] = th*t3[i];
-		  t38[i] = 2.0*th;
-		  t39[i] = pow(t10[i],t38[i]);
-		  t49[i] = pow(t2[i],t38[i]);
-		  t59[i] = pow(t10[i],3.0*th);
-		  t69[i] = t12[i]*t12[i];
-		  t6[i] = 1.0-de*v[i];
-		  t7[i] = pow(t6[i],th);
-		  t8[i] = 1.0-t7[i];
-		  t15[i] = 1.0-(1.0-t3[i])*t8[i]*t13[i];
-		  t17[i] = pow(t15[i],t16[i]);
-		  t25[i] = t3[i]*t7[i];
-		  t26[i] = t11[i]-t7[i]-t3[i]+t25[i];
-		  t29[i] = pow(-t26[i]/t12[i],t16[i]);
-		  t44[i] = pow(t6[i],t38[i]);
-		  t45[i] = t3[i]*t44[i];
-		  t50[i] = t49[i]*t7[i];
-		  t54[i] = t49[i]*t44[i];
-		  t62[i] = -2.0*t25[i]*t11[i]+t25[i]-t33[i]*t7[i]+3.0*t33[i]*t7[i]*t11[i]-3.0*t33[i]*t7[i]*t39[i]+t25[i]*t39[i]+
-			  2.0* t45[i]*t11[i]-t45[i]*t39[i]+2.0*t50[i]*t11[i]-t50[i]*t39[i]-2.0*t54[i]*t11[i]+t54[i]*t39[i]+t54[i]-
-			  t50[i]-t45[i]+t33[i]*t7[i]*t59[i];
-		  t67[i] = t26[i]*t26[i];
-		  out[i] = -de*t29[i]*t62[i]/t6[i]/t2[i]/t67[i]/t69[i];
+		  t2 = 1.0-de*u[i];
+		  t3 = pow(t2,th);
+		  t4 = 1.0-t3;
+		  t10 = 1.0-de;
+		  t11 = pow(t10,th);
+		  t12 = 1.0-t11;
+		  t13 = 1/t12;
+		  t16 = 1/th;
+		  t33 = th*t3;
+		  t38 = 2.0*th;
+		  t39 = pow(t10,t38);
+		  t49 = pow(t2,t38);
+		  t59 = pow(t10,3.0*th);
+		  t69 = t12*t12;
+		  t6 = 1.0-de*v[i];
+		  t7 = pow(t6,th);
+		  t8 = 1.0-t7;
+		  t15 = 1.0-(1.0-t3)*t8*t13;
+		  t17 = pow(t15,t16);
+		  t25 = t3*t7;
+		  t26 = t11-t7-t3+t25;
+		  t29 = pow(-t26/t12,t16);
+		  t44 = pow(t6,t38);
+		  t45 = t3*t44;
+		  t50 = t49*t7;
+		  t54 = t49*t44;
+		  t62 = -2.0*t25*t11+t25-t33*t7+3.0*t33*t7*t11-3.0*t33*t7*t39+t25*t39+
+			  2.0* t45*t11-t45*t39+2.0*t50*t11-t50*t39-2.0*t54*t11+t54*t39+t54-
+			  t50-t45+t33*t7*t59;
+		  t67 = t26*t26;
+		  out[i] = -de*t29*t62/t6/t2/t67/t69;
 	}
 
-	Free(t2);Free(t3);Free(t12);Free(t16);
-	Free(t6);Free(t7);Free(t8);Free(t10);
-	Free(t11);Free(t13);Free(t15);Free(t17);
-	Free(t33);Free(t38);Free(t39);Free(t49);
-	Free(t59);Free(t69);Free(t25);Free(t26);
-	Free(t29);Free(t44);Free(t45);Free(t50);
-	Free(t54);Free(t62);Free(t67);Free(t4);
 }
 
 
@@ -927,7 +777,7 @@ void LL(int* family, int* n, double* u, double* v, double* theta, double* nu, do
 		for(j=0;j<*n;j++)
 		{
 			dat[0] = 1-u[j]; dat[1] = 1-v[j];
-			dbb7(&dat[0], &dat[1], &k, param, &fuc[j]);
+			dbb8(&dat[0], &dat[1], &k, param, &fuc[j]);
 
 			if(!isfinite(fuc[j]) || isnan(fuc[j]))
 			{
